Module: PWN::SAST::TaskTag
- Defined in:
- lib/pwn/sast/task_tag.rb
Overview
SAST Module used to identify task tags such as TODO, SECURITY, FIXME, etc to ensure developers aren't introducing security-related bugs into source code.
Class Method Summary collapse
-
.authors ⇒ Object
- Author(s)
0day Inc.
-
.help ⇒ Object
Display Usage for this Module.
-
.scan(opts = {}) ⇒ Object
- Supported Method Parameters
PWN::SAST::TaskTag.scan( dir_path: 'optional path to dir defaults to .' git_repo_root_uri: 'optional http uri of git repo scanned' ).
-
.security_references ⇒ Object
Used primarily to map NIST 800-53 Revision 4 Security Controls https://web.nvd.nist.gov/view/800-53/Rev4/impact?impactName=HIGH to PWN Exploit & Static Code Anti-Pattern Matching Modules to Determine the level of Testing Coverage w/ PWN.
Class Method Details
.authors ⇒ Object
- Author(s)
0day Inc. support@0dayinc.com
69 70 71 72 73 |
# File 'lib/pwn/sast/task_tag.rb', line 69 public_class_method def self. "AUTHOR(S): 0day Inc. <support@0dayinc.com> " end |
.help ⇒ Object
Display Usage for this Module
77 78 79 80 81 82 83 84 85 86 |
# File 'lib/pwn/sast/task_tag.rb', line 77 public_class_method def self.help puts "USAGE: sast_arr = #{self}.scan( dir_path: 'optional path to dir defaults to .', git_repo_root_uri: 'optional http uri of git repo scanned' ) #{self}.authors " end |
.scan(opts = {}) ⇒ Object
- Supported Method Parameters
PWN::SAST::TaskTag.scan( dir_path: 'optional path to dir defaults to .' git_repo_root_uri: 'optional http uri of git repo scanned' )
18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 |
# File 'lib/pwn/sast/task_tag.rb', line 18 public_class_method def self.scan(opts = {}) dir_path = opts[:dir_path] git_repo_root_uri = opts[:git_repo_root_uri].to_s.scrub test_case_filter = " grep -Fin \ -e '//' \ -e '<!--' \ -e '/**' \ -e '#' {PWN_SAST_SRC_TARGET} 2> /dev/null | \ grep -i \ -e SECURITY \ -e TODO \ -e FIXME \ -e XXX \ -e CHECKME \ -e DOCME \ -e TESTME \ -e PENDING \ -e DISCUSS | grep -v '://' " PWN::SAST::TestCaseEngine.execute( test_case_filter: test_case_filter, security_references: security_references, dir_path: dir_path, git_repo_root_uri: git_repo_root_uri ) rescue StandardError => e raise e end |
.security_references ⇒ Object
Used primarily to map NIST 800-53 Revision 4 Security Controls https://web.nvd.nist.gov/view/800-53/Rev4/impact?impactName=HIGH to PWN Exploit & Static Code Anti-Pattern Matching Modules to Determine the level of Testing Coverage w/ PWN.
55 56 57 58 59 60 61 62 63 64 65 |
# File 'lib/pwn/sast/task_tag.rb', line 55 public_class_method def self.security_references { sast_module: self, section: 'LEAST PRIVILEGE', nist_800_53_uri: 'https://csrc.nist.gov/projects/cprt/catalog#/cprt/framework/version/SP_800_53_5_1_1/home?element=AC-06', cwe_id: '546', cwe_uri: 'https://cwe.mitre.org/data/definitions/546.html' } rescue StandardError => e raise e end |