Module: PWN::SAST::UseAfterFree
- Defined in:
- lib/pwn/sast/use_after_free.rb
Overview
SAST Module used to identify banned function calls in C & C++ code per: https://msdn.microsoft.com/en-us/library/bb288454.aspx
Class Method Summary collapse
-
.authors ⇒ Object
- Author(s)
0day Inc.
-
.help ⇒ Object
Display Usage for this Module.
-
.scan(opts = {}) ⇒ Object
- Supported Method Parameters
PWN::SAST::UseAfterFree.scan( :dir_path => 'optional path to dir defaults to .' :git_repo_root_uri => 'optional http uri of git repo scanned' ).
-
.security_references ⇒ Object
Used primarily to map NIST 800-53 Revision 4 Security Controls https://web.nvd.nist.gov/view/800-53/Rev4/impact?impactName=HIGH to PWN Exploit & Static Code Anti-Pattern Matching Modules to Determine the level of Testing Coverage w/ PWN.
Class Method Details
.authors ⇒ Object
- Author(s)
0day Inc. [email protected]
62 63 64 65 66 |
# File 'lib/pwn/sast/use_after_free.rb', line 62 public_class_method def self. "AUTHOR(S): 0day Inc. <[email protected]> " end |
.help ⇒ Object
Display Usage for this Module
70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 |
# File 'lib/pwn/sast/use_after_free.rb', line 70 public_class_method def self.help puts "USAGE: # :dir_path => optional path to dir defaults to . #{self}.scan( dir_path: 'optional - dir path value consumed by #scan', git_repo_root_uri: 'optional - git repo root uri value consumed by #scan' ) # Used primarily to map NIST 800-53 Revision 4 Security Controls #{self}.security_references # Print the AUTHOR(S) string for this module. #{self}.authors " constants.sort end |
.scan(opts = {}) ⇒ Object
- Supported Method Parameters
PWN::SAST::UseAfterFree.scan( :dir_path => 'optional path to dir defaults to .' :git_repo_root_uri => 'optional http uri of git repo scanned' )
18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 |
# File 'lib/pwn/sast/use_after_free.rb', line 18 public_class_method def self.scan(opts = {}) dir_path = opts[:dir_path] git_repo_root_uri = opts[:git_repo_root_uri].to_s.scrub test_case_filter = " grep -Fn \ -e 'calloc(' \ -e 'free(' \ -e 'malloc(' \ -e 'realloc(' {PWN_SAST_SRC_TARGET} 2> /dev/null " include_extensions = %w[.c .cats .idc .cpp .cc .cxx .c++ .cp .CPP .C .cppm .ixx .h .hpp .hxx .hh .h++ .inc .inl .ipp .tcc .tpp .txx .i .s .asm .o .obj .a .so .lib .dll .exe .pdb .vcxproj .sln .dsp .dsw .cbp .cmake .make .mk] PWN::SAST::TestCaseEngine.execute( test_case_filter: test_case_filter, security_references: security_references, dir_path: dir_path, include_extensions: include_extensions, git_repo_root_uri: git_repo_root_uri ) rescue StandardError => e raise e end |
.security_references ⇒ Object
Used primarily to map NIST 800-53 Revision 4 Security Controls https://web.nvd.nist.gov/view/800-53/Rev4/impact?impactName=HIGH to PWN Exploit & Static Code Anti-Pattern Matching Modules to Determine the level of Testing Coverage w/ PWN.
48 49 50 51 52 53 54 55 56 57 58 |
# File 'lib/pwn/sast/use_after_free.rb', line 48 public_class_method def self.security_references { sast_module: self, section: 'MEMORY PROTECTION', nist_800_53_uri: 'https://csrc.nist.gov/projects/cprt/catalog#/cprt/framework/version/SP_800_53_5_1_1/home?element=SI-16', cwe_id: '416', cwe_uri: 'https://cwe.mitre.org/data/definitions/416.html' } rescue StandardError => e raise e end |