Module: PWN::AI::Anthropic
- Defined in:
- lib/pwn/ai/anthropic.rb
Overview
This plugin interacts with Anthropic’s Claude API. It provides methods to list models, generate completions, and chat. API documentation: docs.anthropic.com/en/api Obtain an API key from console.anthropic.com/
Class Method Summary collapse
-
.authors ⇒ Object
- Author(s)
-
0day Inc.
-
.chat(opts = {}) ⇒ Object
- Supported Method Parameters
-
response = PWN::AI::Anthropic.chat( request: ‘required - message to Anthropic’, model: ‘optional - model to use for text generation (defaults to PWN::Env[:anthropic])’, temp: ‘optional - creative response float (defaults to PWN::Env[:anthropic])’, system_role_content: ‘optional - context to set up the model behavior for conversation (Default: PWN::Env[:anthropic])’, response_history: ‘optional - pass response back in to have a conversation’, speak_answer: ‘optional speak answer using PWN::Plugins::Voice.text_to_speech (Default: nil)’, timeout: ‘optional timeout in seconds (defaults to 300)’, spinner: ‘optional - display spinner (defaults to false)’ ).
-
.get_models ⇒ Object
- Supported Method Parameters
-
models = PWN::AI::Anthropic.get_models.
-
.help ⇒ Object
Display Usage for this Module.
Class Method Details
.authors ⇒ Object
- Author(s)
-
0day Inc. <support@0dayinc.com>
253 254 255 256 257 |
# File 'lib/pwn/ai/anthropic.rb', line 253 public_class_method def self. "AUTHOR(S): 0day Inc. <support@0dayinc.com> " end |
.chat(opts = {}) ⇒ Object
- Supported Method Parameters
-
response = PWN::AI::Anthropic.chat(
request: 'required - message to Anthropic', model: 'optional - model to use for text generation (defaults to PWN::Env[:ai][:anthropic][:model])', temp: 'optional - creative response float (defaults to PWN::Env[:ai][:anthropic][:temp])', system_role_content: 'optional - context to set up the model behavior for conversation (Default: PWN::Env[:ai][:anthropic][:system_role_content])', response_history: 'optional - pass response back in to have a conversation', speak_answer: 'optional speak answer using PWN::Plugins::Voice.text_to_speech (Default: nil)', timeout: 'optional timeout in seconds (defaults to 300)', spinner: 'optional - display spinner (defaults to false)')
147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 |
# File 'lib/pwn/ai/anthropic.rb', line 147 public_class_method def self.chat(opts = {}) engine = PWN::Env[:ai][:anthropic] request = opts[:request] max_prompt_length = engine[:max_prompt_length] ||= 200_000 request_trunc_idx = ((max_prompt_length - 1) / 3.36).floor request = request[0..request_trunc_idx] model = opts[:model] ||= engine[:model] raise 'ERROR: Model is required. Call #get_models method for details' if model.nil? temp = opts[:temp].to_f ||= engine[:temp].to_f temp = 1 if temp.zero? rest_call = 'messages' response_history = opts[:response_history] system_role_content = opts[:system_role_content] ||= engine[:system_role_content] system_role_content = response_history[:choices].first[:content] if response_history && response_history[:choices] system_role = { role: 'system', content: system_role_content } user_role = { role: 'user', content: request } response_history ||= { choices: [system_role] } http_body = { model: model, max_tokens: 4096, temperature: temp, system: system_role_content, messages: [] } if response_history[:choices].length > 1 response_history[:choices][1..].each do || next if [:role] == 'system' http_body[:messages].push(role: [:role].to_s, content: [:content].to_s) end end http_body[:messages].push(role: 'user', content: request) timeout = opts[:timeout] spinner = opts[:spinner] response = anthropic_rest_call( http_method: :post, rest_call: rest_call, http_body: http_body, timeout: timeout, spinner: spinner ) json_resp = JSON.parse(response, symbolize_names: true) assistant_content = if json_resp[:content] && json_resp[:content].is_a?(Array) && json_resp[:content].first json_resp[:content].first[:text] else '' end assistant_resp = { role: 'assistant', content: assistant_content } # Build choices for PWN compatibility: [system, ...history..., user, assistant] json_resp[:choices] = [system_role] + http_body[:messages] json_resp[:choices].push(assistant_resp) # Ensure compatibility fields json_resp[:id] ||= "msg_#{SecureRandom.hex(8)}" json_resp[:object] ||= 'message' json_resp[:model] ||= model if json_resp[:usage].is_a?(Hash) inp_tokens = json_resp[:usage][:input_tokens] || 0 out_tokens = json_resp[:usage][:output_tokens] || 0 json_resp[:usage][:total_tokens] = inp_tokens + out_tokens else json_resp[:usage] = { input_tokens: 0, output_tokens: 0, total_tokens: 0 } end speak_answer = true if opts[:speak_answer] if speak_answer answer = assistant_resp[:content] text_path = "/tmp/#{SecureRandom.hex}.pwn_voice" File.write(text_path, answer) PWN::Plugins::Voice.text_to_speech(text_path: text_path) File.unlink(text_path) end json_resp rescue StandardError => e raise e end |
.get_models ⇒ Object
- Supported Method Parameters
-
models = PWN::AI::Anthropic.get_models
127 128 129 130 131 132 133 |
# File 'lib/pwn/ai/anthropic.rb', line 127 public_class_method def self.get_models models = anthropic_rest_call(rest_call: 'models') JSON.parse(models, symbolize_names: true)[:data] rescue StandardError => e raise e end |
.help ⇒ Object
Display Usage for this Module
261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 |
# File 'lib/pwn/ai/anthropic.rb', line 261 public_class_method def self.help puts "USAGE: models = #{self}.get_models response = #{self}.chat( request: 'required - message to Anthropic', model: 'optional - model to use for text generation (defaults to PWN::Env[:ai][:anthropic][:model])', temp: 'optional - creative response float (defaults to PWN::Env[:ai][:anthropic][:temp])', system_role_content: 'optional - context to set up the model behavior for conversation (Default: PWN::Env[:ai][:anthropic][:system_role_content])', response_history: 'optional - pass response back in to have a conversation', speak_answer: 'optional speak answer using PWN::Plugins::Voice.text_to_speech (Default: nil)', timeout: 'optional - timeout in seconds (defaults to 300)', spinner: 'optional - display spinner (defaults to false)' ) #{self}.authors " end |